
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "kxsldbg_part.h"
#include "libxsldbg/files.h"
#include "libxsldbg/xsldbg.h"

#include <kcomponentdata.h>
#include <kaction.h>
#include <kstandardaction.h>
#include <kfiledialog.h>
#include <kparts/genericfactory.h>
#include <ktexteditor/markinterface.h>
#include <ktexteditor/editor.h>

#include <qfile.h>
#include <qtextstream.h>

#include "../kxsldbg.h"
#include <kaction.h>
#include <kcmdlineargs.h>
#include <kcomponentdata.h>
#include <kiconloader.h>
#include <qmessagebox.h>
#include <klocale.h>
#include <kdeversion.h>
#include <kinputdialog.h>

// Qxsldbg specific includes
#include "qxsldbgdoc.h"
#include <qvariant.h>
#include <qfile.h>
#include <qstatusbar.h>
#include <qsplitter.h>



#include <qmime.h>
#include <q3dragobject.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <q3whatsthis.h>
#include <qaction.h>
#include <qmenubar.h>
#include <q3popupmenu.h>
#include <q3toolbar.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtextstream.h>
#include <q3textbrowser.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <q3dockwindow.h>
#include <qpushbutton.h>
#include <qinputdialog.h>
#include <qobject.h>
#include <QStackedWidget>
#include "xsldbgoutputview.h"
#include "xsldbgconfigimpl.h"
#include <kdebug.h>
#include <kvbox.h>
#include <kactioncollection.h>
typedef KParts::GenericFactory<KXsldbgPart> KXsldbgPartFactory;
K_EXPORT_COMPONENT_FACTORY( libkxsldbgpart, KXsldbgPartFactory )

KXsldbgPart::KXsldbgPart( QWidget *parentWidget,
                                  QObject *parent,
                                  const QStringList & /*args*/ )
  : KParts::ReadOnlyPart(parent)
{
    currentLineNo = 0;
    currentColumnNo = 0;
    inspector = 0L;
    debugger = 0L;
    configWidget = 0L;
    currentDoc = 0L;

    // we need an instance
    setComponentData( KXsldbgPartFactory::componentData() );
    KVBox *frame = new KVBox(parentWidget);
    KHBox *h = new KHBox(frame);
    newXPath = new QLineEdit(h);
    xPathBtn = new QPushButton(i18n("Goto XPath"), h);
/* Disable searching as searching documentation is not ready
    h = new QHBox(frame);
    newSearch = new QLineEdit(h);
    searchBtn = new QPushButton(i18n("Search"), h);
*/
    h = new KHBox(frame);
    newEvaluate = new QLineEdit(h);
    evaluateBtn = new QPushButton(i18n("Evaluate"), h);

    QSplitter *splitter = new QSplitter(Qt::Vertical, frame);
    mainView = new QStackedWidget(splitter);
    outputview = new XsldbgOutputView(splitter);
    setWidget(frame);
    docDictionary.setAutoDelete(true);

    // create our actions
    KStandardAction::open(this, SLOT(fileOpen()), actionCollection());

    // set our XML-UI resource file
   setXMLFile("kxsldbg_part.rc");
   QAction *action ;
   action = actionCollection()->addAction( "configureEditorCmd" );
   action->setText( i18n("Configure Editor...") );
   action->setIcon( KIcon("configure") );
   connect(action, SIGNAL(triggered(bool)), SLOT(configureEditorCmd_activated()));
   action = actionCollection()->addAction( "configureCmd" );
   action->setText( i18n("Configure...") );
   action->setIcon( KIcon("configure") );
   connect(action, SIGNAL(triggered(bool)), SLOT(configureCmd_activated()));
   action->setShortcut(QKeySequence(Qt::Key_C));

   action = actionCollection()->addAction( "inspectCmd" );
   action->setText( i18n("Inspect...") );
   action->setIcon( KIcon("edit-find") );
    connect(action, SIGNAL(triggered(bool)), SLOT(inspectorCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_I));


    // Motions commands
    action = actionCollection()->addAction( "runCmd" );
    action->setText( i18n("Run") );
    action->setIcon( KIcon("system-run") );

    connect(action, SIGNAL(triggered(bool)), SLOT(runCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F5));

    action = actionCollection()->addAction( "continueCmd" );
    action->setText( i18n("Continue") );
    action->setIcon( KIcon("arrow-down") );

    connect(action, SIGNAL(triggered(bool)), SLOT(continueCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F4));

    action = actionCollection()->addAction( "stepCmd" );
    action->setText( i18n("Step") );
    action->setIcon( KIcon("step") );

    connect(action, SIGNAL(triggered(bool)), SLOT(stepCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F8));

    action  = new KAction(KIcon("find-next"), i18n("Next"), this);
    actionCollection()->addAction("nextCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(nextCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F10));

    action  = new KAction(KIcon("xsldbg_stepup"), i18n("Step Up"), this);
    actionCollection()->addAction("stepupCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(stepupCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F6));

    action  = new KAction(KIcon("xsldbg_stepdown"), i18n("Step Down"), this);
    actionCollection()->addAction("stepdownCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(stepCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F7));

    // Breakpoint commands
    action  = new KAction(KIcon("xsldbg_break"), i18n("Break"), this);
    actionCollection()->addAction("breakCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(breakCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F2));

    action  = new KAction(KIcon("xsldbg_enable"), i18n("Enable/Disable"), this);
    actionCollection()->addAction("enableCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(enableCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_F3));

    action  = new KAction(KIcon("xsldbg_delete"), i18n("Delete"), this);
    actionCollection()->addAction("deleteCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(deleteCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_Delete));

    action  = new KAction(KIcon("xsldbg_source"), i18n("&Source"), this);
    actionCollection()->addAction("sourceCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(sourceCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_S));

    action  = new KAction(KIcon("xsldbg_data"), i18n("&Data"), this);
    actionCollection()->addAction("dataCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(dataCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_D));

    action  = new KAction(KIcon("xsldbg_output"), i18n("&Output"), this);
    actionCollection()->addAction("outputCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(outputCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_O));

    action  = new KAction(KIcon("xsldbg_refresh"), i18n("Reload Current File From Disk"), this);
    actionCollection()->addAction("refreshCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(refreshCmd_activated()));
    action->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_F5));

    /* tracing and walking */
    action  = new KAction(i18n("Walk Through Stylesheet..."), this);
    actionCollection()->addAction("walkCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(walkCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_W));
    action  = new KAction(i18n("Stop Wal&king Through Stylesheet"), this);
    actionCollection()->addAction("walkStopCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(walkStopCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_K));
    action  = new KAction(i18n("Tr&ace Execution of Stylesheet"), this);
    actionCollection()->addAction("traceCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(traceCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_A));
    action  = new KAction(i18n("Stop Tracing of Stylesheet"), this);
    actionCollection()->addAction("traceStopCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(traceStopCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_K));

    action  = new KAction(i18n("&Evaluate Expression..."), this);
    actionCollection()->addAction("evaluateCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(evaluateCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_E));

    action  = new KAction(i18n("Goto &XPath..."), this);
    actionCollection()->addAction("gotoXPathCmd", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(gotoXPathCmd_activated()));
    action->setShortcut(QKeySequence(Qt::Key_X));

    action  = new KAction(i18n("Lookup SystemID..."), this);
    actionCollection()->addAction("lookupSystemID", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(slotLookupSystemID()));

    action  = new KAction(i18n("Lookup PublicID..."), this);
    actionCollection()->addAction("lookupPublicID", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(slotLookupPublicID()));

    action  = new KAction(i18n("Quit"), this);
    actionCollection()->addAction("file_quit", action );
    connect(action, SIGNAL(triggered(bool)), SLOT(quit()));
    action->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));

    /*
    action = new KAction( i18n("Exit KXsldbg"),
                        "xsldbg_output", actionCollection(), "exitCmd" );
    connect(action, SIGNAL(triggered(bool)), SLOT(exitCmd_activated()));
    action->setShortcut(QKeySequence(CTRL + Key_Q));
    */
    connect( xPathBtn, SIGNAL( clicked() ),
             this, SLOT( slotGotoXPath() ) );
    connect( evaluateBtn, SIGNAL( clicked() ),
             this, SLOT( slotEvaluate() ) );
/*

    connect( searchBtn, SIGNAL( clicked() ),
             this, SLOT( slotSearch() ) );
*/
/* We must have a valid debugger and inspector */
    createInspector();
    if (checkDebugger()){
	configWidget = new XsldbgConfigImpl( debugger, 0L );
	Q_CHECK_PTR( configWidget );
	debugger->start();
    }else{
	openUrl(KUrl());
    }
}

KXsldbgPart::~KXsldbgPart()
{
    docDictionary.clear();
}

void KXsldbgPart::quit()
{
    qWarning("Custom void KXsldbgPart::quit()");
    closeUrl();
}


KAboutData *KXsldbgPart::createAboutData()
{
    // the non-i18n name here must be the same as the directory in
    // which the part's rc file is installed ('partrcdir' in the
    // Makefile)
    KAboutData *aboutData = new KAboutData("kxsldbgpart", I18N_NOOP("KXsldbgPart"), "0.1");
    aboutData->addAuthor("Keith Isdale", 0L, "k_isdale@tpg.com.au");
    return aboutData;
}


bool  KXsldbgPart::openUrl(const KUrl &url)
{
    bool result = fetchURL(url);
    if (result){
	QXsldbgDoc *docPtr = docDictionary[url.prettyUrl()];
	if (docPtr && docPtr->kateView()){
	    if (docPtr != currentDoc){
		currentDoc = docPtr;
		currentFileName = url.prettyUrl();
		mainView->setCurrentWidget(currentDoc->kateView());
		emit setWindowCaption(currentDoc->url().prettyUrl());
	    }
	} else{
	    result = false;
	}
    }

    return result;
}


/* Don't show the content of URL just loaded it into our data structures */
bool  KXsldbgPart::fetchURL(const KUrl &url)
{
    QString docID = url.prettyUrl();
    QXsldbgDoc *docPtr = docDictionary[docID];
    if (!docPtr){
	docPtr = new QXsldbgDoc(mainView, url);
	docDictionary.insert(docID, docPtr);
	if (docPtr->kateView()){
#ifdef __GNUC__
#warning "kde4: port it"
#endif
#if 0
		mainView->addWidget(docPtr->kateView());
		KTextEditor::View *v = KTextEditor::View((docPtr->kateView()));
		connect(v, SIGNAL(cursorPositionChanged()), this, SLOT(cursorPositionChanged()));
#endif
	}
    }

    return true;
}

bool KXsldbgPart::openFile()
{
    qWarning("bool KXsldbgPart::openFile() called");
    return false;
}

bool KXsldbgPart::closeUrl()
{
    docDictionary.clear();
    return true;
}

void KXsldbgPart::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
    QString file_name = KFileDialog::getOpenFileName();

    if (file_name.isEmpty() == false)
        openUrl(KUrl( file_name ));
}

void KXsldbgPart::configureEditorCmd_activated()
{
    if (currentDoc){
      currentDoc->kateDoc()->editor()->configDialog(mainView);
    }
}

bool KXsldbgPart::checkDebugger()
{
  bool result = debugger != 0L;
  if (!result){
    QMessageBox::information(0L, i18n("Debugger Not Ready"),
			      i18n("Configure and start the debugger first."),
			      QMessageBox::Ok);
  }

  return result;
}


void KXsldbgPart::lookupSystemID( QString systemID)
{
   bool ok = false;
   if (!checkDebugger())
     return;

   if (systemID.isEmpty()){
     systemID = KInputDialog::getText(
			      i18n( "Lookup SystemID" ),
			      i18n( "Please enter SystemID to find:" ),
			      QString::null, &ok,
			      mainView);
   }else{
     ok = true;
   }
  if ( ok && !systemID.isEmpty() ){
      // user entered something and pressed ok
	QString msg(QString("system %1").arg(systemID));  // noTr
	debugger->fakeInput(msg, true);
  }

}


void KXsldbgPart::lookupPublicID(QString publicID)
{
   bool ok = false;
   if (!checkDebugger())
     return;

   if (publicID.isEmpty()){
     publicID = KInputDialog::getText(
			      i18n( "Lookup PublicID" ),
			      i18n( "Please enter PublicID to find:" ),
			      QString::null, &ok, mainView );
   }else{
     ok = true;
   }
   if ( ok && !publicID.isEmpty()){
     // user entered something and pressed ok
	QString msg(QString("public %1").arg(publicID));  // noTr
	debugger->fakeInput(msg, true);
  }
}


void KXsldbgPart::slotLookupSystemID()
{
  lookupSystemID("");
}

void KXsldbgPart::slotLookupPublicID()
{
  lookupPublicID("");
}

void KXsldbgPart::configureCmd_activated()
{
  if (!checkDebugger())
    return;

  if (configWidget != 0L){
    configWidget->refresh();
    configWidget->show();
  }
}

void
KXsldbgPart::runCmd_activated()
{
    if ( checkDebugger() )
        debugger->slotRunCmd();
}

void KXsldbgPart::inspectorCmd_activated()
{
  if (inspector == 0L)
    createInspector();

  if (checkDebugger() && (inspector != 0L)){
    inspector->show();
  }
}


void KXsldbgPart::createInspector()
{
    if ( inspector == 0L ) {
      debugger = new XsldbgDebugger();
      Q_CHECK_PTR( debugger );
      if ( debugger != 0L ) {
	       connect(debugger, SIGNAL( debuggerReady()),
		     this, SLOT(debuggerStarted()));
	       if (outputview){
		 connect(debugger,
			 SIGNAL( showMessage(QString /* msg*/)),
			 outputview,
		  SLOT(slotProcShowMessage(QString /* msg*/)));
	       }
	      inspector = new XsldbgInspector( debugger );
	      Q_CHECK_PTR( inspector );
	      debugger->setInspector( inspector );
	      if (inspector != 0L){
		/*process line number and/or file name changed */
		connect(debugger,
			SIGNAL(lineNoChanged
				(QString /* fileName */ ,
				 int /* lineNumber */ ,
				 bool /* breakpoint */ ) ),
			this,
			SLOT(lineNoChanged
			     ( QString /* fileName */ ,
			       int /* lineNumber */ ,
			       bool /* breakpoint */ ) ) );
		connect(debugger,
			SIGNAL(breakpointItem(QString /* fileName*/,
					      int /* lineNumber */,
					      QString /*templateName*/,
					      QString /* modeName */,
					      bool /* enabled */,
					      int /* id */)),
			this,
			SLOT( breakpointItem(QString /* fileName*/,
					     int /* lineNumber */,
					     QString /*templateName*/,
					     QString /* modeName */,
					     bool /* enabled */,
					     int /* id */)));
		connect(debugger, SIGNAL(resolveItem(QString /*URI*/)),
			this, SLOT(slotProcResolveItem(QString /*URI*/)));
	      }
        }
    }
}

void KXsldbgPart::emitOpenFile(QString file, int line, int row)
{
    QByteArray params;
    QDataStream stream(&params, QIODevice::WriteOnly);
    stream << file << line << row;
#ifdef __GNUC__
#warning "Port to DBUS!"
#endif
//FIXME Port to DBUS!
//    emitDCOPSignal("openFile(QString,int,int)", params);
}
void KXsldbgPart::continueCmd_activated()
{
    if ( checkDebugger() )
        debugger->slotContinueCmd();

}

void KXsldbgPart::stepCmd_activated()
{
    if ( checkDebugger() )
        debugger->slotStepCmd();
}

void KXsldbgPart::nextCmd_activated()
{
    if ( checkDebugger() )
        debugger->fakeInput("next", true);  // noTr
}


void KXsldbgPart::stepupCmd_activated()
{
    if ( checkDebugger() )
      debugger->fakeInput("stepup", true);  // noTr
}


void KXsldbgPart::stepdownCmd_activated()
{
    if ( checkDebugger() )
      debugger->fakeInput("stepdown", true);  // noTr
}


void KXsldbgPart::dataCmd_activated()
{
    if ( checkDebugger() )
        debugger->slotDataCmd();
}

void
KXsldbgPart::sourceCmd_activated()
{
    if ( checkDebugger() )
        debugger->slotSourceCmd();
}

void
KXsldbgPart::outputCmd_activated()
{
  if ( ( inspector != 0L ) &&  checkDebugger() && ( configWidget != 0L ) ){
    debugger->setOutputFileActive(true);
    lineNoChanged( configWidget->getOutputFile(), 1, false );
    refreshCmd_activated();
  }
}

void KXsldbgPart::refreshCmd_activated()
{

  if ( !currentFileName.isEmpty() ){
    Q3DictIterator<QXsldbgDoc> it(docDictionary);
    QXsldbgDoc *docPtr;
    while (it.current()){
      docPtr = it.current();
      docPtr->refresh();
     ++it;
    }
    if ( checkDebugger() ){
	debugger->fakeInput("showbreak", true);  // noTr
    }
  }
}

void KXsldbgPart::enableCmd_activated()
{
  if ( checkDebugger() ){
    debugger->slotEnableCmd( currentFileName, currentLineNo);
  }
}

void KXsldbgPart::deleteCmd_activated()
{
  if ( checkDebugger() ){
        debugger->slotDeleteCmd( currentFileName, currentLineNo);
  }
}

void KXsldbgPart::breakCmd_activated()
{
  if ( checkDebugger() ){
    debugger->slotBreakCmd( currentFileName, currentLineNo);
  }
}

void KXsldbgPart::evaluateCmd_activated()
{
#if KDE_IS_VERSION(3,1,90)
  QString expression = KInputDialog::getText(i18n("Evaluate Expression"), i18n("XPath:"));
#else
  QString expression = KLineEditDlg::getText(i18n("Evaluate Expression"), i18n("XPath:"));
#endif
  if (checkDebugger()  && (expression.length() > 0)){
    debugger->slotCatCmd( expression);
  }
}

void KXsldbgPart::gotoXPathCmd_activated()
{
#if KDE_IS_VERSION(3,1,90)
  QString xpath = KInputDialog::getText(i18n("Goto XPath"), i18n("XPath:"));
#else
  QString xpath = KLineEditDlg::getText(i18n("Goto XPath"), i18n("XPath:"));
#endif
  if (checkDebugger() && xpath.length() > 0){
    debugger->slotCdCmd( xpath );
  }
}

void
KXsldbgPart::lineNoChanged(QString fileName, int lineNumber, bool breakpoint)
{
  if ( fileName.isEmpty() ) {
    kDebug() << "Empty file Name" << endl;  // noTr
    return;
  }

  openUrl(fileName);

 /* Did we stop at a breakpoint if so move the marker */
  if (breakpoint && currentDoc) {
      currentDoc->selectBreakPoint(lineNumber);
      QByteArray params;
      QDataStream stream(&params, QIODevice::WriteOnly);
      stream << currentFileName << lineNumber;
#ifdef __GNUC__
#warning "Port to DBUS!"
#endif
//FIXME Port to DBUS!
//      emitDCOPSignal("debuggerPositionChanged(QString,int)", params);
  }

  /* Move cursor and update status bar */
  if (currentDoc && currentDoc->kateView()){
    currentDoc->kateView()->setCursorPosition(KTextEditor::Cursor(lineNumber - 1, 0));
	  currentLineNo = lineNumber;
  }
}

void KXsldbgPart::cursorPositionChanged()
{
    if (currentDoc && currentDoc->kateView()){
      KTextEditor::Cursor c = currentDoc->kateView()->cursorPosition();
      currentLineNo = c.line();
      currentColumnNo = c.column();
	    currentLineNo++;
	    currentColumnNo++;
	    QByteArray params;
	    QDataStream stream(&params, QIODevice::WriteOnly);
	    stream << currentFileName << currentLineNo << currentColumnNo;
#ifdef __GNUC__
#warning "Port to DBUS!"
#endif
//FIXME Port to DBUS!
//	    emitDCOPSignal("editorPositionChanged(QString,int,int)", params);
    }
}

void  KXsldbgPart::docChanged()
{
    if (!currentDoc || currentDoc->kateDoc() || currentDoc->kateView())
	return;
}

void  KXsldbgPart::debuggerStarted()
{
    if (configWidget != 0L){
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if (args){
	    int i=0, result=1, noFilesFound = 0;
	    QString expandedName;      /* contains file name with path expansion if any */

	    for (i = 0; i < args->count(); i++) {
		if (!result)
		    break;

		if (args->arg(i)[0] != '-') {
		    expandedName = QString::fromUtf8((const char*)filesExpandName((const xmlChar*)args->arg(i)));
		    if (expandedName.isEmpty()) {
			result = 0;
			break;
		    }
		    switch (noFilesFound) {
			case 0:
			    configWidget->slotSourceFile(expandedName);
			    noFilesFound++;
			    break;
			case 1:
			    configWidget->slotDataFile(expandedName);
			    noFilesFound++;
			    break;
			case 2:
			    configWidget->slotOutputFile(expandedName);
			    noFilesFound++;
			    break;

			default:
			    xsldbgGenericErrorFunc(i18n("Error: Too many file names supplied via command line.\n"));
			    result = 0;
		    }
		    continue;
		}
	    }
	    configWidget->refresh();
	    configWidget->show();
	}
    }
}

void  KXsldbgPart::addBreakPoint(int lineNumber)
{
  if ( checkDebugger() ){
    debugger->slotBreakCmd( currentFileName, lineNumber);
  }
}

void  KXsldbgPart::enableBreakPoint(int lineNumber)
{
  if ( checkDebugger() ){
    debugger->slotEnableCmd( currentFileName, lineNumber);
  }
}


void  KXsldbgPart::deleteBreakPoint(int lineNumber)
{
  if ( checkDebugger() ){
        debugger->slotDeleteCmd( currentFileName, lineNumber);
  }
}



void KXsldbgPart::slotSearch()
{
  if ((newSearch != 0L)  && checkDebugger() ) {
    QString msg(QString("search \"%1\"").arg(newSearch->text()));  // noTr
 		debugger->fakeInput(msg, false);
  }
}


void KXsldbgPart::slotEvaluate()
{
  if ((newEvaluate != 0L) && checkDebugger() ){
    debugger->slotCatCmd( newEvaluate->text() );
  }
}

void KXsldbgPart::slotGotoXPath()
{
  if ((newXPath != 0L) && checkDebugger() ){
    debugger->slotCdCmd( newXPath->text() );
  }
}



void KXsldbgPart::slotProcResolveItem(QString URI)
{
  if (!URI.isEmpty()){
    QMessageBox::information(mainView, i18n("SystemID or PublicID Resolution Result"),
        i18n("SystemID or PublicID has been resolved to\n.%1", URI),
	QMessageBox::Ok);
  }
}

void  KXsldbgPart::breakpointItem(QString fileName, int lineNumber ,
			      QString /*templateName*/, QString /* modeName */,
			      bool  enabled , int /* id */)
{

  if (fileName == 0L){
    /* Go through all documents and remove all breakpoints */
    Q3DictIterator<QXsldbgDoc> it(docDictionary);
#ifdef __GNUC__
#warning "kde4: port it"
#endif
#if 0
    QXsldbgDoc *docPtr;
    KTextEditor::MarkInterface *markIf = 0L;
    while (it.current()){
      docPtr = it.current();
      markIf = KTextEditor::markInterface(docPtr->kateDoc());
      if (markIf)
	   markIf->clearMarks();
     ++it;
    }
#endif
  }else{
/*
    if (!fileName.contains("://")){
	// relative path ? must handle this special case
	KUrl url;
	url.setFileName(fileName);
	fetchURL(url);
    }else{
	fetchURL(fileName);
    }
*/
    fetchURL(fileName);
    QXsldbgDoc *docPtr = docDictionary[fileName] ;
   if (docPtr)
	docPtr->enableBreakPoint(--lineNumber, enabled);
  }
}



void KXsldbgPart::walkCmd_activated()
{
  if (checkDebugger()){
    debugger->slotWalkCmd();
  }
}

void KXsldbgPart::walkStopCmd_activated()
{
  if (checkDebugger()){
    debugger->slotWalkStopCmd();
  }
}

void KXsldbgPart::traceCmd_activated()
{
  if (checkDebugger()){
    debugger->slotTraceCmd();
  }
}

void KXsldbgPart::traceStopCmd_activated()
{
  walkStopCmd_activated();
}


#include "kxsldbg_part.moc"
