/**
   File : qxsldbgdoc.cpp
   Author : Keith Isdale
   Date : 19th April 2002
   Description : The document to handle source and breakpoints
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ************************************************************************* */

#include "qxsldbgdoc.h"
#include <ktexteditor/markinterface.h>
#include <ktexteditor/editorchooser.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>

QXsldbgDoc::QXsldbgDoc(QWidget *parent, KUrl url)
    : QObject(0L, "QXsldbgDoc"), kDoc(0L),kView(0L), locked(false) 
{
	KTextEditor::Editor *editor = KTextEditor::EditorChooser::editor();


    kDoc = editor->createDocument( 0L);
    connect(kDoc, SIGNAL(started(KIO::Job *)), this, SLOT(lockDoc()));
    connect(kDoc, SIGNAL(completed()), this, SLOT(unlockDoc()));
    if (kDoc){
	kView = qobject_cast<KTextEditor::View*>(kDoc->createView(parent));// "QXsldbgDocView");
	KUrl cleanUrl;
	// convert paths relative to PWD into a absolute path
	QString relUrl = url.prettyUrl();
	if (!relUrl.contains(":/")){
	    if (!(relUrl.left(1) == "/"))
		relUrl.prepend(QDir::currentPath() + '/');
	    cleanUrl.setFileName(relUrl); 
	}else{
	    cleanUrl = url; 
	}
	kDoc->openUrl(cleanUrl);
    }
}


QXsldbgDoc::~QXsldbgDoc()
{
    if (kDoc){
	if (kDoc->views().count() == 1){
	    	kDoc->closeUrl(false);	
		delete kDoc;
	}
    }
}

void QXsldbgDoc::slotResult( KJob *job )
{
    if ( job->error() != 0 ){
    }else{
    }
    emit docChanged();
}

KUrl QXsldbgDoc::url() const
{
    if (kDoc)
	return kDoc->url();
    else
	return KUrl();
}

void QXsldbgDoc::refresh()
{
    if (kDoc){
	KUrl lastUrl = kDoc->url();	
	kDoc->closeUrl(false);
	kDoc->openUrl(lastUrl);
    }
}


void QXsldbgDoc::enableBreakPoint(uint lineNumber, bool state)
{
    if (locked)
	return;
#ifdef __GNUC__
#warning "kde4: port it"	
#endif
#if 0
    KTextEditor::MarkInterface *markIf = KTextEditor::markInterface(kDoc);
    if (markIf){
	if (state)
	    markIf->setMark(lineNumber, KTextEditor::MarkInterface::markType02);
	else
	    markIf->setMark(lineNumber, KTextEditor::MarkInterface::markType03);
    }
#endif	
}

void QXsldbgDoc::addBreakPoint(uint lineNumber)
{
    enableBreakPoint(lineNumber, true);    
}

void QXsldbgDoc::deleteBreakPoint(uint lineNumber)
{
    if (locked)
	return;
#ifdef __GNUC__
#warning "kde4: port it"	
#endif
#if 0
    KTextEditor::MarkInterface *markIf = KTextEditor::markInterface(kDoc);
    if (markIf)
	markIf->clearMark(lineNumber);
#endif	
}


void QXsldbgDoc::selectBreakPoint(uint lineNumber)
{
    if (locked)
	return;
#ifdef __GNUC__
#warning "kde4: port it"	
#endif
#if 0
    KTextEditor::MarkInterface *markIf = KTextEditor::markInterface(kDoc);
    if (markIf)
	markIf->setMark(lineNumber, KTextEditor::MarkInterface::markType05);
#endif	
}

void QXsldbgDoc::lockDoc()
{
    locked = true;
}

void QXsldbgDoc::unlockDoc()
{
    locked = false;
}
#include "qxsldbgdoc.moc"
